@@ -1,6 +1,6 @@
 package Date::Hijri::Simple;
 
-$Date::Hijri::Simple::VERSION = '0.02';
+$Date::Hijri::Simple::VERSION = '0.03';
 
 =head1 NAME
 
@@ -8,13 +8,15 @@ Date::Hijri::Simple - Represents Hijri date.
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
 use 5.006;
 use Data::Dumper;
 use Time::localtime;
+use List::Util qw/min/;
+use POSIX qw/floor ceil/;
 
 use Moo;
 use namespace::clean;
@@ -27,11 +29,37 @@ Represents the Hijri date.
 
 =cut
 
+our $HIJRI_MONTHS = [
+    undef,
+    q/Muharram/, q/Safar/ , q/Rabi' al-awwal/, q/Rabi' al-thani/,
+    q/Jumada al-awwal/, q/Jumada al-thani/, q/Rajab/ , q/Sha'aban/,
+    q/Ramadan/ , q/Shawwal/ , q/Dhu al-Qi'dah/ , q/Dhu al-Hijjah/
+];
+
+our $HIJRI_DAYS = [
+    '<yellow><bold>      al-Ahad </bold></yellow>',
+    '<yellow><bold>   al-Ithnayn </bold></yellow>',
+    '<yellow><bold> ath-Thulatha </bold></yellow>',
+    '<yellow><bold>     al-Arbia </bold></yellow>',
+    '<yellow><bold>    al-Khamis </bold></yellow>',
+    '<yellow><bold>    al-Jumuah </bold></yellow>',
+    '<yellow><bold>      as-Sabt </bold></yellow>',
+];
+
+our $HIJRI_LEAP_YEAR_MOD = [
+    2, 5, 7, 10, 13, 16, 18, 21, 24, 26, 29
+];
+
+has hijri_epoch         => (is => 'ro', default => sub { 1948439.5            });
+has hijri_days          => (is => 'ro', default => sub { $HIJRI_DAYS          });
+has hijri_months        => (is => 'ro', default => sub { $HIJRI_MONTHS        });
+has hijri_leap_year_mod => (is => 'ro', default => sub { $HIJRI_LEAP_YEAR_MOD });
+
 has year  => (is => 'rw', predicate => 1);
 has month => (is => 'rw', predicate => 1);
 has day   => (is => 'rw', predicate => 1);
 
-with 'Date::Utils::Hijri';
+with 'Date::Utils';
 
 sub BUILD {
     my ($self) = @_;
@@ -45,10 +73,10 @@ sub BUILD {
         my $year  = $today->year + 1900;
         my $month = $today->mon + 1;
         my $day   = $today->mday;
-        my ($y, $m, $d) = $self->gregorian_to_hijri($year, $month, $day);
-        $self->year($y);
-        $self->month($m);
-        $self->day($d);
+        my $date  = $self->from_gregorian($year, $month, $day);
+        $self->year($date->year);
+        $self->month($date->month);
+        $self->day($date->day);
     }
 }
 
@@ -60,8 +88,8 @@ sub BUILD {
     # prints today's hijri date
     print Date::Hijri::Simple->new, "\n";
 
-    my $date = Date::Hijri::Simple->new({ year => 1436, month => 1, day => 1 });
-    print "Date: $date\n";
+    # prints given hijri date
+    print Date::Hijri::Simple->new({ year => 1436, month => 1, day => 1 })->as_string
 
     # prints equivalent Julian date
     print $date->to_julian, "\n";
@@ -83,7 +111,33 @@ Returns julian date equivalent of the Hijri date.
 sub to_julian {
     my ($self) = @_;
 
-    return $self->hijri_to_julian($self->year, $self->month, $self->day);
+    return ($self->day + ceil(29.5 * ($self->month - 1))
+            + ($self->year - 1) * 354
+            + floor((3 + (11 * $self->year)) / 30)
+            + $self->hijri_epoch) - 1;
+}
+
+=head2 from_julian($julian_date)
+
+Returns Hijri  date as an object of type L<Date::Hijri::Simple> equivalent of the
+given Julian date C<$julian_date>.
+
+=cut
+
+sub from_julian {
+    my ($self, $julian_date) = @_;
+
+    $julian_date = floor($julian_date) + 0.5;
+    my $year     = floor(((30 * ($julian_date - $self->hijri_epoch)) + 10646) / 10631);
+    my $a_hijri  = Date::Hijri::Simple->new({ year => $year, month => 1, day => 1 });
+    my $month    = min(12, ceil(($julian_date - (29 + $a_hijri->to_julian)) / 29.5) + 1);
+    my $b_hijri  = Date::Hijri::Simple->new({ year => $year, month => $month, day => 1 });
+    my $day      = ($julian_date - $b_hijri->to_julian) + 1;
+
+    return Date::Hijri::Simple->new({
+        year  => $year,
+        month => $month,
+        day   => $day });
 }
 
 =head2 to_gregorian()
@@ -99,6 +153,19 @@ sub to_gregorian {
     return sprintf("%04d-%02d-%02d", $date[0], $date[1], $date[2]);
 }
 
+=head2 from_gregorian($year, $month, $day)
+
+Returns Hijri  date as an object of type L<Date::Hijri::Simple> equivalent of the
+Gregorian date C<$year>, C<$month> and C<$day>.
+
+=cut
+
+sub from_gregorian {
+    my ($self, $year, $month, $day) = @_;
+
+    return $self->from_julian($self->gregorian_to_julian($year, $month, $day));
+}
+
 =head2 day_of_week()
 
 Returns day of the week, starting 0 for al-Ahad, 1 for al-Ithnayn and so on.
@@ -123,6 +190,62 @@ sub day_of_week {
     return $self->jwday($self->to_julian);
 }
 
+=head2 is_hijri_leap_year($year)
+
+Returns 0 or 1 if the given Hijri year C<$year> is a leap year or not.
+
+=cut
+
+sub is_hijri_leap_year {
+    my ($self, $year) = @_;
+
+    my $mod = $year % 30;
+    return 1 if grep/$mod/, @{$self->hijri_leap_year_mod};
+    return 0;
+}
+
+=head2 days_in_hijri_year($year)
+
+Returns the number of days in the given year of Hijri Calendar.
+
+=cut
+
+sub days_in_hijri_year {
+    my ($self, $year) = @_;
+
+    ($self->is_hijri_leap_year($year))
+    ?
+    (return 355)
+    :
+    (return 354);
+}
+
+=head2 days_in_hijri_month_year($month, $year)
+
+Returns total number of days in the given Hijri month year.
+
+=cut
+
+sub days_in_hijri_month_year {
+    my ($self, $month, $year) = @_;
+
+    return 30 if (($month % 2 == 1) || (($month == 12) && ($self->is_hijri_leap_year($year))));
+    return 29;
+}
+
+=head2 validate_day($day)
+
+Dies if the given C<$day> is not a valid Hijri day.
+
+=cut
+
+sub validate_day {
+    my ($self, $day) = @_;
+
+    die("ERROR: Invalid day [$day].\n")
+        unless (defined($day) && ($day =~ /^\d{1,2}$/) && ($day >= 1) && ($day <= 30));
+}
+
 sub as_string {
     my ($self) = @_;
 
